package com.muyu.common.cache;


import com.muyu.common.cache.data.MysqlPrefix;
import com.muyu.data.source.domain.DataSource;
import lombok.Builder;
import lombok.Data;


@Data
@Builder
public class MysqlPoolConfig extends MysqlPrefix {
    /**
     * id
     */
    private String id;
    /**
     * 链接类型
     */
    private Long dataAccessTypeId;
    /**
     * 初始化连接数
     */
    private Integer initTotal;
    /**
     * 最大连接数
     */
    private Integer maxTotal;
    /**
     * 最长等待时间 毫秒
     */
    private Integer maxWaitTimes;
    /**
     * 驱动
     */
    private String driverName;
    /**
     * url ip 端口 数据库名 编码
     */
    private String ip;
    private String port;
    private String databaseName;
    private String param;
    /**
     * 用户名
     */
    private String userName;
    /**
     * 密码
     */

    private String password;

    public static MysqlPoolConfig buildConfig(DataSource sysDataSource) {
        return MysqlPoolConfig.builder()
                .id(String.valueOf(sysDataSource.getId()))
                .dataAccessTypeId(sysDataSource.getDataAccessTypeId())
                .initTotal(sysDataSource.getInitialQuantity())
                .maxTotal(sysDataSource.getMaximumQuantity())
                .maxWaitTimes(sysDataSource.getMaximumTime())
                .driverName("com.mysql.cj.jdbc.Driver")
                .ip(sysDataSource.getHostAddress())
                .port(sysDataSource.getHostPort())
                .databaseName(sysDataSource.getDatabaseName())
                .param(sysDataSource.getDataConnectionParameter())
                .userName(sysDataSource.getDatabaseUserName())
                .password(sysDataSource.getDatabaseUserPassword())
                .build();
    }


    /**
     * 获取数据库连接
     */
    public String getUrl() {
        StringBuilder urlSb = new StringBuilder(MysqlPrefix.MYSQLDUMP);
        urlSb.append(this.ip);
        urlSb.append(":");
        urlSb.append(this.port);
        urlSb.append("/");
        urlSb.append(this.databaseName);
        urlSb.append("?");
        urlSb.append(this.param);
        return urlSb.toString();
    };

    public String getPostgreSqlUrl() {
        StringBuilder urlSb = new StringBuilder(MysqlPrefix.POSTGRESQL);
        urlSb.append(this.ip);
        urlSb.append(":");
        urlSb.append(this.port);
        urlSb.append("/");
        urlSb.append(this.databaseName);
        return urlSb.toString();
    };

}
