package com.muyu.method.SQL.implementation;

import com.muyu.method.SQL.SqlMethod;
import org.springframework.stereotype.Component;


import java.util.Map;


/**
 * SQL实现层
 *
 * @ClassName SqlMethodImplementation
 * @Author AnNan.Wang
 * @Date 2024/5/12 14:29
 */

@Component
public class SqlMethodImplementation implements SqlMethod<String, Object, String> {
    // 插入SQL接口
    @Override
    public String insertData(String key, Map<String, Object> value) {

        StringBuilder columns = new StringBuilder();
        StringBuilder values = new StringBuilder();

        for (String column : value.keySet()) {
            columns.append(column).append(",");
            values.append("?,");
        }
        //删除最后一个逗号
        columns.deleteCharAt(columns.length() - 1);
        values.deleteCharAt(values.length() - 1 );

        String sql = "INSERT INTO " + key + " (" + columns + ") VALUES (" +  values + ")";
        return sql;
    }
    // 查询SQL接口
    @Override
    public String selectData(String key,Map<String, Object> value ,String condition) {
        StringBuilder sql = new StringBuilder("SELECT * FROM ").append(key);

        // 如果有条件，构建 WHERE 子句
        if (condition != null && !condition.isEmpty()) {
            StringBuilder whereClause = new StringBuilder(" WHERE ");
            int index = 0;
            for (String column : value.keySet()) {
                whereClause.append(column).append(" = ?");
                // 如果不是最后一个条件，则添加 AND 连接符
                if (index < value.size() - 1) {
                    whereClause.append(" AND ");
                }
                index++;
            }
            sql.append(whereClause);
        }

        return sql.toString();
    }
    // 修改SQL接口
    @Override
    public String updateData(String key, Map<String, Object> value, String condition, Map<String, Object> updateData) {
        StringBuilder setClause = new StringBuilder();
        for (String column : value.keySet()) {
            setClause.append(column).append("=?,");
        }

        // 删除最后一个逗号
        setClause.deleteCharAt(setClause.length() - 1);

        String sql = "UPDATE " + key + " SET " + setClause;
        if (condition != null && !condition.isEmpty()) {
            sql += " WHERE ";
        }
        int index = 0;
        StringBuilder builder = new StringBuilder();
        for (String column : updateData.keySet()) {
            builder.append(column).append(" = ?");
            if (index < value.size() - 1) {
                builder.append(" AND ");
            }
            index++;
        }
        sql += builder;
        return sql;

    }
    // 删除SQL接口
    @Override
    public String deleteData(String key, Map<String, Object> value, String condition) {
        StringBuilder whereClause = new StringBuilder();
        String sql = "DELETE FROM " + key;

        // 添加自定义条件
        if (condition != null && !condition.isEmpty()) {
            whereClause.append(" WHERE ").append(condition);
        }

        // 添加条件参数
        if (value != null && !value.isEmpty()) {
            if (whereClause.length() == 0) {
                whereClause.append(" WHERE ");
            } else {
                whereClause.append(" AND ");
            }
            int index = 0;
            for (String column : value.keySet()) {
                whereClause.append(column).append(" = ?");
                if (index < value.size() - 1) {
                    whereClause.append(" AND ");
                }
                index++;
            }
        }

        sql += whereClause.toString();
        return sql;
    }

}
