package com.muyu.method.druid;

import com.alibaba.druid.pool.DruidDataSource;
import com.muyu.data.source.domain.DataSource;
import jdk.jfr.Label;
import lombok.extern.log4j.Log4j2;
import org.bouncycastle.crypto.macs.HMac;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.HashMap;

/**
 * druid 德鲁伊连接池配置
 *
 * @author AnNan.Wang
 * @ClassName: druidPort
 * @createTime: 2024/5/16 14:11
 */
@Log4j2
public class druidPort{

    private static HashMap<String, DruidDataSource> dataSourceMap = new HashMap<>();


    //初始化数据库连接池
    public  static void init(DataSource dataSource){
        DruidDataSource druidDataSource = new DruidDataSource();
        //用户名
        druidDataSource.setUsername(dataSource.getDatabaseUserName());
        //密码
        druidDataSource.setPassword(dataSource.getDatabaseUserPassword());

        String name =null;

        if (dataSource.getDataAccessTypeId() == 1) {
            name = "jdbc:mysql://";
        }else {
            name = "jdbc:postgresql://";
        }

        //URL
        druidDataSource.setUrl(name+dataSource.getHostAddress()+":"+dataSource.getHostPort()+"/"+dataSource.getDatabaseName());

        //初始连接数
        druidDataSource.setInitialSize(dataSource.getInitialQuantity());
        //最大活动连接数
        druidDataSource.setMaxActive(dataSource.getMaximumQuantity());
        //最小空闲连接数
        druidDataSource.setMinIdle(dataSource.getMaximumFrequency());
        //最大等待时间
        druidDataSource.setMaxWait(dataSource.getMaximumTime());

        try {
            druidDataSource.init();
        } catch (SQLException e) {
            log.info("初始化数据源失败");
        }
        dataSourceMap.put(dataSource.getAccessSourceName()+dataSource.getId(),druidDataSource);
    }

    //获取连接池信息
    public static void getNum(String key){
        DruidDataSource druidDataSource = dataSourceMap.get(key);
        log.info(key + "正在使用连接" + druidDataSource.getActiveCount() + "个,线程数量:" + druidDataSource.getPoolingCount() + "个");

    }



    public static Connection getConnection(String key){
        DruidDataSource druidDataSource = dataSourceMap.get(key);
        if (druidDataSource != null) {
            try {
                return druidDataSource.getConnection();
            } catch (SQLException e) {
                throw new RuntimeException(e);
            }
        }
        return null;
    }

    public static  void close(Connection connection){
        try {
            connection.close();
        } catch (SQLException e) {
            log.error("释放连接失败");
        }
    }
}
