package com.muyu.source.clinet.factory;

import com.muyu.data.source.domain.DataSource;
import lombok.Data;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

@Data
public class DataSourceSingleton{
    public Map<String, DataSource> dataSourceMap = new HashMap<>();

    public static volatile DataSourceSingleton instance;

    // 使用双重检查锁确保线程安全
    public static DataSourceSingleton getInstance() {
        if (instance == null) {
            synchronized (DataSourceSingleton.class) {
                if (instance == null) {
                    instance = new DataSourceSingleton();
                }
            }
        }
        return instance;
    }

    // 添加数据源
    public void addDataSource(String key, DataSource dataSource) {
        dataSourceMap.put(key, dataSource);
    }

    // 获取数据源
    public DataSource getDataSource(String key) {
        return dataSourceMap.get(key);
    }
}
