package com.muyu.data.source.domain;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.muyu.common.core.web.domain.BaseEntity;
import com.muyu.data.source.domain.req.AssetAuthInfoQueryReq;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import lombok.experimental.SuperBuilder;

import java.util.Date;
import java.util.function.Supplier;

/**
 * 资产授权对象
 *
 * @ClassName asset_authInfo
 * @Author AnNan.Wang
 * @Date 2024/5/5 15:28
 */

@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
@TableName(value = "asset_auth_info")
@EqualsAndHashCode(callSuper = true)
@ApiModel(value = "AssetAuthInfo",description = "资产授权")
public class AssetAuthInfo extends BaseEntity {
    private static final long serialVersionUID = 1L;

    /** 授权主键 */
    @TableId(value = "id",type = IdType.AUTO)
    @ApiModelProperty(name = "授权主键",value = "授权主键")
    private Long id;

    /** 授权编号 */
    @ApiModelProperty(name = "授权编号",value = "授权编号")
    private Long authId;

    /** 编号类型 dept/user */
    @ApiModelProperty(name = "编号类型 dept/user",value = "编号类型 dept/user")
    private String idType;

    /** 授权数据 */
    @ApiModelProperty(name = "授权数据",value = "授权数据")
    private String authData;

    /** 授权类型 datasource/database/table */
    @ApiModelProperty(name = "授权类型 datasource/database/table",value = "授权类型 datasource/database/table")
    private String authType;

    public static AssetAuthInfo queryBuild(AssetAuthInfoQueryReq assetAuthInfoQueryReq) {
        return AssetAuthInfo.builder()
                .authId(assetAuthInfoQueryReq.getAuthId())
                .idType(assetAuthInfoQueryReq.getIdType())
                .authData(assetAuthInfoQueryReq.getAuthData())
                .authType(assetAuthInfoQueryReq.getAuthType())
                .build();
    }

    public static AssetAuthInfo saveBuild(AssetAuthInfoQueryReq assetAuthInfoQueryReq, Supplier<String> getUsername) {

        return AssetAuthInfo.builder()
                .authId(assetAuthInfoQueryReq.getAuthId())
                .idType(assetAuthInfoQueryReq.getIdType())
                .authData(assetAuthInfoQueryReq.getAuthData())
                .authType(assetAuthInfoQueryReq.getAuthType())
                .createBy(getUsername.get())
                .createTime(new Date())
                .build();
    }
}
