package com.muyu.data.source.service.impl;

import java.sql.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.function.Function;
import java.util.stream.Collectors;

import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.ObjUtils;
import com.muyu.data.source.domain.*;
import com.muyu.data.source.domain.model.DataTableValue;
import com.muyu.data.source.domain.model.DatabaseTableModel;
import com.muyu.data.source.domain.red.DataSourceTableRed;
import com.muyu.data.source.domain.req.DataSourceSaveReq;
import com.muyu.data.source.domain.req.DatabaseConnect;
import com.muyu.data.source.mapper.AssetMapper;
import com.muyu.data.source.mapper.DatabaseTableInformationMapper;
import com.muyu.data.source.mapper.DatabaseTableMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.muyu.data.source.mapper.DataSourceMapper;
import com.muyu.data.source.service.DataSourceService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;

/**
 * 数据源Service业务层处理
 *
 * @author muyu
 * @date 2024-04-21
 */
@Slf4j
@Service
public class DataSourceServiceImpl extends ServiceImpl<DataSourceMapper, DataSource>  implements DataSourceService {
    @Autowired
    private DataSourceMapper dataSourceMapper;
    /**
     * 查询数据源列表
     *
     * @param dataSource 数据源
     * @return 数据源
     */
    @Override
    public List<DataSource> list(DataSource dataSource) {
        LambdaQueryWrapper<DataSource> queryWrapper = new LambdaQueryWrapper<>();


        if (ObjUtils.notNull(dataSource.getAccessSourceName())){
            queryWrapper.like(DataSource::getAccessSourceName, dataSource.getAccessSourceName());
        }

        if (ObjUtils.notNull(dataSource.getDataSourceSystemName())){
            queryWrapper.like(DataSource::getDataSourceSystemName, dataSource.getDataSourceSystemName());
        }

        if (ObjUtils.notNull(dataSource.getHostAddress())){
            queryWrapper.eq(DataSource::getHostAddress, dataSource.getHostAddress());
        }

        if (ObjUtils.notNull(dataSource.getHostPort())){
            queryWrapper.eq(DataSource::getHostPort, dataSource.getHostPort());
        }

        if (ObjUtils.notNull(dataSource.getDataAccessTypeId())){
            queryWrapper.eq(DataSource::getDataAccessTypeId, dataSource.getDataAccessTypeId());
        }

        if (ObjUtils.notNull(dataSource.getDatabaseName())){
            queryWrapper.like(DataSource::getDatabaseName, dataSource.getDatabaseName());
        }

        if (ObjUtils.notNull(dataSource.getDataConnectionParameter())){
            queryWrapper.eq(DataSource::getDataConnectionParameter, dataSource.getDataConnectionParameter());
        }

        if (ObjUtils.notNull(dataSource.getInitialQuantity())){
            queryWrapper.eq(DataSource::getInitialQuantity, dataSource.getInitialQuantity());
        }

        if (ObjUtils.notNull(dataSource.getMaximumQuantity())){
            queryWrapper.eq(DataSource::getMaximumQuantity, dataSource.getMaximumQuantity());
        }



        if (ObjUtils.notNull(dataSource.getMaximumFrequency())){
            queryWrapper.eq(DataSource::getMaximumFrequency, dataSource.getMaximumFrequency());
        }





        return list(queryWrapper);
    }
    @Autowired
    private AssetMapper assetMapper;

    @Override
    public List<AccessType> findAccessType() {
        return dataSourceMapper.findAccessType();
    }

    private final static String postgresSqlTable = "SELECT \n" +
            "            TABLE_NAME AS 'tableName',\n" +
            "                COLUMN_NAME AS 'name',\n" +
            "                COLUMN_COMMENT AS 'comment',\n" +
            "                CASE WHEN COLUMN_KEY = 'PRI' THEN '是' ELSE '否' END AS 'isPrimaryKey',\n" +
            "\n" +
            "                CASE\n" +
            "                    WHEN DATA_TYPE = 'int' THEN 'Integer'\n" +
            "                    WHEN DATA_TYPE = 'varchar' THEN 'String'\n" +
            "                    WHEN DATA_TYPE = 'decimal' THEN 'BigDecimal'\n" +
            "                    WHEN DATA_TYPE = 'tinyint' AND COLUMN_TYPE = 'tinyint(1)' THEN 'Boolean'\n" +
            "\n" +
            "                    ELSE DATA_TYPE\n" +
            "                    END AS 'type',\n" +
            "                DATA_TYPE AS 'mappingType',\n" +
            "                COLUMN_TYPE AS 'detailType',\n" +
            "                CHARACTER_MAXIMUM_LENGTH AS 'length',\n" +
            "                NUMERIC_SCALE AS 'decimalPlaces',\n" +
            "                IS_NULLABLE AS 'isNull',\n" +
            "                COLUMN_DEFAULT AS 'defaultValue'\n" +
            "        FROM\n" +
            "            INFORMATION_SCHEMA.COLUMNS\n";
    @Override
    public Result testConnection(DataSourceSaveReq dataSourceQueryReq) {
        String user = dataSourceQueryReq.getDatabaseUserName();
        String password = dataSourceQueryReq.getDatabaseUserPassword();
        String url = "jdbc:mysql://"+dataSourceQueryReq.getHostAddress()+":"+dataSourceQueryReq.getHostPort()+"/"+dataSourceQueryReq.getDatabaseName();
        ArrayList<String> arrayList = new ArrayList<>();
        //数据展示ArrayList集合
        ArrayList<Asset> assetArrayList = new ArrayList<>();
        try {
            Connection con = DriverManager.getConnection(url, user, password);
            if (con!=null){
                Statement statement = con.createStatement();

                String query = "SHOW TABLES";

                ResultSet resultSet = statement.executeQuery(query);

                while (resultSet.next()) {
                    String string = resultSet.getString(1);
                    arrayList.add(string);
                }

                //数据库表对象ArrayList集合
                ArrayList<DatabaseTable> tableArrayList = new ArrayList<>();
                //查询出表结构SQL
                String mysql = postgresSqlTable+
                        "        WHERE\n" +
                        "            TABLE_SCHEMA = ?\n" +
                        "          AND TABLE_NAME = ?";
                arrayList.forEach(res->{
                    try {
                        PreparedStatement preparedStatement = con.prepareStatement(mysql);
                        preparedStatement.setString(1,dataSourceQueryReq.getDatabaseName());
                        preparedStatement.setString(2,res);

                        ResultSet executeQuery = preparedStatement.executeQuery();
                        while (executeQuery.next()) {
                            DatabaseTable databaseTable = new DatabaseTable();
                            databaseTable.setTableName(res);
                            databaseTable.setName( executeQuery.getString("name"));
                            databaseTable.setComment( executeQuery.getString("comment"));
                            databaseTable.setIsPrimaryKey( executeQuery.getString("isPrimaryKey"));
                            databaseTable.setType(executeQuery.getString("type"));
                            databaseTable.setDetailType( executeQuery.getString("mappingType"));
                            databaseTable.setLength( executeQuery.getLong("length"));
                            databaseTable.setDecimalPlaces( executeQuery.getDouble("decimalPlaces"));
                            databaseTable.setIsNull( executeQuery.getString("isNull"));
                            databaseTable.setDefaultValue( executeQuery.getString("defaultValue"));
                            databaseTable.setDatabaseName(dataSourceQueryReq.getDatabaseName());

                            String selectedSql = "SELECT * FROM " +res;

                            Statement conStatement = con.createStatement();
                            ResultSet executedQuery = conStatement.executeQuery(selectedSql);
                            ResultSetMetaData metaData = executedQuery.getMetaData();
                            int columnCount = metaData.getColumnCount();



                            while (executedQuery.next()) {
                                for (int i = 1; i <= columnCount; i++) {
                                    Asset asset = new Asset();

                                    String catalogName = metaData.getColumnName(i);
                                    String key =dataSourceQueryReq.getAccessSourceName()+"-"+res+"-"+catalogName;

                                    asset.setAssetKey(key);

                                    asset.setValue(executedQuery.getObject(i));

                                    asset.setType(metaData.getColumnTypeName(i));

                                    asset.setDatabaseId(dataSourceQueryReq.getId());
                                    asset.setTableName(res);
                                    assetArrayList.add(asset);
                                }
                            }

                        }

                    } catch (SQLException e) {
                        throw new RuntimeException(e);
                    }
                });
                List<Asset> uniqueAssets = assetArrayList.stream()
                    .collect(Collectors.toMap(
                            asset -> Arrays.asList(asset.getAssetKey(), asset.getType(), asset.getValue(), asset.getDatabaseId(), asset.getTableName()),
                            Function.identity(),
                            (existing, replacement) -> existing,
                            LinkedHashMap::new))
                    .values() // 获取值的集合，即去重后的 Asset 对象集合
                    .stream() // 转换回 Stream
                    .toList();
                for (Asset uniqueAsset : uniqueAssets) {
                    assetMapper.insert(uniqueAsset);
                }
                return Result.success("连接成功");

            }
        } catch (Exception e) {
           log.error("连接失败："+e.getMessage());
            return Result.error("连接失败");
        }

      return Result.error("连接失败");
    }

    @Override
    public Result synchronization(DataSource dataSource) {
        AssetStructure assetStructure = new AssetStructure();
        String user=dataSource.getDatabaseUserName();
        String password=dataSource.getDatabaseUserPassword();
        String jdbcDriver="com.mysql.cj.jdbc.Driver";
        String url = "jdbc:mysql://" + dataSource.getHostAddress() + ":" + dataSource.getHostPort() + "/" + dataSource.getDatabaseName();
        String query="show tables";

        ArrayList<DatabaseTableInformation> arrayList = new ArrayList<>();

        try (Connection con = DriverManager.getConnection(url, user, password)) {
            // 创建 PreparedStatement，并设置参数
            PreparedStatement prepared = con.prepareStatement("SELECT " +
                    " TABLE_NAME AS `name`, " +
                    " TABLE_COMMENT AS `as`, " +
                    " TABLE_ROWS AS `dataTotal` " +
                    " FROM INFORMATION_SCHEMA.TABLES " +
                    " WHERE TABLE_SCHEMA = ?");
            prepared.setString(1, dataSource.getDatabaseName()); // 设置数据库名称参数



            // 执行查询
            ResultSet resultSet = prepared.executeQuery();

            ResultSetMetaData rsmd = resultSet.getMetaData();
            // 5. 处理查询结果
            while (resultSet.next()) {
                //表名
                String tableName = resultSet.getString("name");
                //表备注
                String tableComment = resultSet.getString("as");
                //表条数
                int tableRows = resultSet.getInt("dataTotal");
                //查询出表结构

                PreparedStatement preparedStatement = con.prepareStatement(" SELECT " +
                        "    TABLE_NAME AS 'tableName', " +
                        "        COLUMN_NAME AS 'name', " +
                        "        COLUMN_COMMENT AS 'comment', " +
                        "        CASE WHEN COLUMN_KEY = 'PRI' THEN '是' ELSE '否' END AS 'isPrimaryKey', " +
                        "        CASE " +
                        "            WHEN DATA_TYPE = 'int' THEN 'Integer' " +
                        "            WHEN DATA_TYPE = 'varchar' THEN 'String' " +
                        "            WHEN DATA_TYPE = 'decimal' THEN 'BigDecimal' " +
                        "            WHEN DATA_TYPE = 'tinyint' AND COLUMN_TYPE = 'tinyint(1)' THEN 'Boolean' " +
                        "            ELSE DATA_TYPE " +
                        "            END AS 'type', " +
                        "        DATA_TYPE AS 'mappingType', " +
                        "        COLUMN_TYPE AS 'detailType', " +
                        "        CHARACTER_MAXIMUM_LENGTH AS 'length', " +
                        "        NUMERIC_SCALE AS 'decimalPlaces', " +
                        "        IS_NULLABLE AS 'isNull', " +
                        "        COLUMN_DEFAULT AS 'defaultValue' " +
                        "FROM " +
                        "    INFORMATION_SCHEMA.COLUMNS " +
                        "WHERE " +
                        "    TABLE_SCHEMA = ? " +
                        "  AND TABLE_NAME = ?");

                preparedStatement.setString(1,dataSource.getDatabaseName());
                preparedStatement.setString(2,tableName);
                ResultSet executed = preparedStatement.executeQuery();

                List<DatabaseTable> databaseTables = new ArrayList<>();
                while (executed.next()) {
                    DatabaseTable databaseTable = new DatabaseTable();
                    databaseTable.setTableName(tableName);
                    databaseTable.setName( executed.getString("name"));
                    databaseTable.setComment( executed.getString("comment"));
                    databaseTable.setIsPrimaryKey( executed.getString("isPrimaryKey"));
                    databaseTable.setType( executed.getString("type"));
                    databaseTable.setDetailType( executed.getString("mappingType"));
                    databaseTable.setLength( executed.getLong("length"));
                    databaseTable.setDecimalPlaces( executed.getDouble("decimalPlaces"));
                    databaseTable.setIsNull( executed.getString("isNull"));
                    databaseTable.setDefaultValue( executed.getString("defaultValue"));
                    databaseTable.setDatabaseName(dataSource.getDatabaseName());
                    databaseTables.add(databaseTable);
                }
                //添加进入数据库
                dataSourceMapper.DatabaseTableAdd(databaseTables);

                // 创建 DatabaseTableInformation对象
                DatabaseTableInformation build = DatabaseTableInformation.builder()
                        .name(tableName)
                        .as(tableComment)
                        .dataTotal(tableRows)
                        .build();

                // 将对象添加到 ArrayList 中
                arrayList.add(build);
            }

            assetStructure.setAccessSourceName(dataSource.getAccessSourceName());
            assetStructure.setDatabaseName(dataSource.getDatabaseName());
            assetStructure.setDataSourceSystemName(dataSource.getDataSourceSystemName());

            assetStructure.setDatabaseTableName(rsmd.getTableName(1));

            assetStructure.setDataVolume(arrayList.size());
            Integer num = 0;
            for (DatabaseTableInformation databaseTableInformation : arrayList) {
                 num+=databaseTableInformation.getDataTotal();
            }
            System.out.println(num);
            assetStructure.setDataQuantity(num);

            dataSourceMapper.addAssets1(assetStructure);

            Integer id = assetStructure.getId();

            dataSourceMapper.addDatabaseTableInformation(arrayList,id,dataSource.getDatabaseName());
            return Result.success("同步成功");

        } catch (SQLException e) {
            System.out.println("连接失败"+e.getMessage());
            return Result.error("同步失败");
        }

    }

    @Override
    public Result<List<DatabaseTableModel>> findDataBaseTable(DatabaseConnect databaseConnect) {
        List<DatabaseTableModel> information = dataSourceMapper.DatabaseTableList(databaseConnect.getDatabaseName(),databaseConnect.getName());
        return Result.success(information);
    }



    @Override
    public Result<Structure> findStructure() {
        Structure structure = new Structure();
        List<AssetStructure> assetStructure = dataSourceMapper.findAssetStructure();
        structure.setAssetStructureList(assetStructure);
        return Result.success(structure);
    }


    @Override
    public Result<Structure> structureList(Integer id) {
        List<DatabaseTableInformation> information = dataSourceMapper.findInformation(id);
        Structure structure = new Structure();
        structure.setDatabaseTableInformationList(information);
        return Result.success(structure);
    }


    @Override
    public Result<List<DatabaseTableModel>> database(String name) {
        List<DatabaseTableModel> information = dataSourceMapper.database(name);
        return Result.success(information);
    }


    @Override
    public Result<List<DatabaseTableInformation>> table(String databaseName) {
        List<DatabaseTableInformation> information = dataSourceMapper.table(databaseName);
        return Result.success(information);
    }


    @Override
    public Result<List<DatabaseTableInformation>> quantity() {
        return Result.success(dataSourceMapper.quantity());
    }


    @Autowired
    private DatabaseTableInformationMapper databaseTableInformationMapper;
    @Autowired
    private DatabaseTableMapper databaseTableMapper;

    @Override
    public Result<List<DataSourceTableRed>> information() {
        List<DataSourceTableRed> arrayList = new ArrayList<>();
        List<DataSource> dataSources = dataSourceMapper.selectList(null);
        ArrayList<DatabaseTableInformation> informationArrayList = new ArrayList<>();
        for (DataSource dataSource : dataSources) {

            List<DatabaseTableInformation> table = dataSourceMapper.table(dataSource.getDatabaseName());

            for (DatabaseTableInformation information : table) {
                List<DatabaseTable> databaseTables = databaseTableMapper.selectList(
                        new LambdaQueryWrapper<DatabaseTable>() {{
                            eq(DatabaseTable::getTableName, information.getName());
                            eq(DatabaseTable::getDatabaseName, information.getDatabaseName());
                        }}
                );

                informationArrayList.add(
                        DatabaseTableInformation.builder()
                                .id(information.getId())
                                .name(information.getName())
                                .as(information.getAs())
                                .dataTotal(information.getDataTotal())
                                .databaseName(information.getDatabaseName())
                                .type(information.getType())
                                .structureId(information.getStructureId())
                                .databaseTables(databaseTables)
                                .build()
                );
            }

            arrayList.add(
                    DataSourceTableRed.builder()
                            .dataSource(dataSource)
                            .databaseTableInformationList(informationArrayList)
                            .build()
            );
        }
        return Result.success(
                arrayList
        );
    }
}
