package com.muyu.data.source.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.domain.Result;
import com.muyu.data.source.domain.SysDictDataTable;
import com.muyu.data.source.domain.SysDictionary;
import com.muyu.data.source.mapper.SysDictDataTableMapper;
import com.muyu.data.source.mapper.SysDictionaryMapper;
import com.muyu.data.source.service.SysDictionaryService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * 字典类型业务实现层
 *
 * @ClassName SysDictionaryServiceImpl
 * @Author AnNan.Wang
 * @Date 2024/4/29 16:12
 */

@Service
public class SysDictionaryServiceImpl extends ServiceImpl<SysDictionaryMapper, SysDictionary> implements SysDictionaryService {
    @Autowired
    private SysDictionaryMapper sysDictionaryMapper;

    @Autowired
    private SysDictDataTableMapper sysDictDataTableMapper;

    @Override
    public Result<List<SysDictionary>> dictionaryList(String databaseName) {

        List<SysDictionary> dictionaryList = sysDictionaryMapper.selectList(
                new LambdaQueryWrapper<SysDictionary>(){{
                    eq(SysDictionary::getTableName, databaseName);
                }}
        );

        List<SysDictionary> arrayList = new ArrayList<>();
        for (SysDictionary sysDictionary : dictionaryList) {
            List<SysDictDataTable> sysDictDataTableList = sysDictDataTableMapper.selectList(
                    new LambdaQueryWrapper<SysDictDataTable>() {{
                        eq(SysDictDataTable::getDictType, sysDictionary.getDictType());
                    }}
            );

            arrayList.add(
                    SysDictionary.builder()
                            .id(sysDictionary.getId())
                            .dictName(sysDictionary.getDictName())
                            .dictType(sysDictionary.getDictType())
                            .status(sysDictionary.getStatus())
                            .sysDictDataTableList(sysDictDataTableList)
                            .build()
            );
        }

        return Result.success(arrayList);
    }

    @Override
    public Result<String> dictionaryAdd(SysDictionary sysDictionary) {
        return Result.success(
                sysDictionaryMapper.insert(
                        SysDictionary.builder()
                                .dictName(sysDictionary.getDictName())
                                .dictType(sysDictionary.getDictType())
                                .remark(sysDictionary.getRemark())
                                .tableName(sysDictionary.getTableName())
                                .build()
                )>0?"添加成功":"添加失败"
        );
    }


    @Override
    public Result<String> dictionaryDelete(String dictType) {
        List<SysDictionary> sysDictionaries = sysDictionaryMapper.selectList(
                new LambdaQueryWrapper<SysDictionary>() {{
                    eq(SysDictionary::getDictType, dictType);
                }}
        );
        for (SysDictionary sysDictionary : sysDictionaries) {
            List<SysDictDataTable> sysDictDataTableList = sysDictDataTableMapper.selectList(
                    new LambdaQueryWrapper<SysDictDataTable>() {{
                        eq(SysDictDataTable::getDictType, sysDictionary.getDictType());
                    }}
            );
            if (sysDictDataTableList.size()>0) {
                int i = sysDictionaryMapper.deleteById(sysDictionary.getId());
                int i1 = sysDictDataTableMapper.deleteBatchIds(sysDictDataTableList);
                if (i>0 && i1>0) {
                    return Result.success("删除成功");
                }else {
                    return Result.error("删除失败");
                }
            } else {
                int i = sysDictionaryMapper.deleteById(sysDictionary.getId());
                if (i>0) {
                    return Result.success("删除成功");
                }else {
                    return Result.error("删除失败");
                }
            }
        }
        return Result.error("删除失败");
    }

    @Override
    public Result<String> dictionaryUpdate(SysDictionary sysDictionary) {
        return Result.success(
                sysDictionaryMapper.updateById(
                        SysDictionary.builder()
                                .id(sysDictionary.getId())
                                .dictName(sysDictionary.getDictName())
                                .dictType(sysDictionary.getDictType())
                                .remark(sysDictionary.getRemark())
                                .build()
                )>0?"修改成功":"修改失败"
        );
    }
}
