package com.muyu.rule.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.muyu.common.core.utils.poi.ExcelUtil;
import com.muyu.common.core.web.page.TableDataInfo;
import com.muyu.rule.domain.EngineVersion;
import com.muyu.rule.domain.Ruleengine;
import com.muyu.rule.domain.model.RuleContentModel;
import com.muyu.rule.domain.red.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.muyu.common.log.annotation.Log;
import com.muyu.common.log.enums.BusinessType;
import com.muyu.common.security.annotation.RequiresPermissions;
import com.muyu.rule.service.IEngineMaintenanceService;
import com.muyu.common.core.web.controller.BaseController;
import com.muyu.common.core.domain.Result;

/**
 * 引擎维护Controller
 *
 * @author wan
 * @date 2024-05-03
 */
@Api(tags = "规则引擎")
@RestController
@RequestMapping("/rule")
public class EngineMaintenanceController extends BaseController
{
    @Autowired
    private IEngineMaintenanceService engineMaintenanceService;

    @PostMapping("/compiler")
    public Result compiler(@RequestBody RuleContentReq ruleContentReq){
        return engineMaintenanceService.compiler(ruleContentReq);
    }

    @PostMapping("/loader")
    public Result loader(@RequestBody RuleContentReq  ruleContentReq){
        return engineMaintenanceService.loader(ruleContentReq);
    }

    @GetMapping("/getRuleContent")
    public Result<RuleContentModel> getRuleContent(@RequestParam Long ruleId) {
        return engineMaintenanceService.getRuleContent(ruleId);
    }
    /**
     * 查询引擎维护列表
     */
    @RequiresPermissions("rule:rule:list")
    @GetMapping("/list")
    public Result<TableDataInfo<Ruleengine>> list(RuleengineQueryReq ruleengineQueryReq)
    {
        startPage();
        List<Ruleengine> list = engineMaintenanceService.list(Ruleengine.queryBuild(ruleengineQueryReq));
        return getDataTable(list);
    }

    /**
     * 导出引擎维护列表
     */
    @RequiresPermissions("rule:rule:export")
    @Log(title = "引擎维护", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, Ruleengine ruleengine)
    {
        List<Ruleengine> list = engineMaintenanceService.list(ruleengine);
        ExcelUtil<Ruleengine> util = new ExcelUtil<Ruleengine>(Ruleengine.class);
        util.exportExcel(response, list, "引擎维护数据");
    }

    /**
     * 获取引擎维护详细信息
     */
    @RequiresPermissions("rule:rule:query")
    @GetMapping(value = "/{id}")
    public Result<Ruleengine> getInfo(@PathVariable("id") Long id)
    {
        return Result.success(engineMaintenanceService.getById(id));
    }

    /**
     * 新增引擎维护
     */
    @RequiresPermissions("rule:rule:add")
    @Log(title = "引擎维护", businessType = BusinessType.INSERT)
    @PostMapping
    public Result<String> add(@RequestBody RuleengineSaveReq engineMaintenance)
    {
        return toAjax(engineMaintenanceService.save(Ruleengine.saveBuild(engineMaintenance)));
    }

    /**
     * 修改引擎维护
     */
    @RequiresPermissions("rule:rule:edit")
    @Log(title = "引擎维护", businessType = BusinessType.UPDATE)
    @PutMapping("/{id}")
    public Result<String> edit(@PathVariable Long id, @RequestBody RuleengineEditReq ruleengineEditReq)
    {
        return toAjax(engineMaintenanceService.updateById(Ruleengine.editBuild(id,ruleengineEditReq)));
    }

    /**
     * 删除引擎维护
     */
    @RequiresPermissions("rule:rule:remove")
    @Log(title = "引擎维护", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    @ApiOperation("删除规则引擎")
    @ApiImplicitParam(name = "id", value = "id", required = true, dataType = "Long", paramType = "path", dataTypeClass = String.class, example = "1,2,3,4")
    public Result remove(@PathVariable List<Long> ids)
    {
        return toAjax(engineMaintenanceService.removeBatchByIds(ids));
    }

    /**
     * 规则维护的详情
     * @param id
     * @return
     */
    @PostMapping("/details")
    public Result<EngineVersionReq> details(@RequestParam("id") Long id){
        return engineMaintenanceService.details(id);
    }

    /**
     * 规则维护的是否激活修改
     * @param ruleengine
     * @return
     */
    @PostMapping("/engineOnOff")
    public Result<String> engineOnOff(@RequestBody Ruleengine ruleengine){
        return engineMaintenanceService.engineOnOff(ruleengine);
    }

    /**
     * 修改规则维护的状态
     * @param ruleengine
     * @return
     */
    @PostMapping("/statusOnOff")
    public Result<String> statusOnOff(@RequestBody Ruleengine ruleengine){
        return engineMaintenanceService.statusOnOff(ruleengine);
    }


}
