package com.muyu.rule.service.impl;

import java.lang.reflect.Method;
import java.net.URI;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.List;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.ObjUtils;
import com.muyu.rule.domain.EngineVersion;
import com.muyu.rule.domain.Ruleengine;
import com.muyu.rule.domain.model.RuleContentModel;
import com.muyu.rule.domain.red.EngineVersionReq;
import com.muyu.rule.domain.red.RuleContentReq;
import com.muyu.rule.mapper.EngineVersionMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.muyu.rule.mapper.EngineMaintenanceMapper;
import com.muyu.rule.service.IEngineMaintenanceService;

import javax.tools.*;

/**
 * 引擎维护Service业务层处理
 *
 * @author wan
 * @date 2024-05-03
 */
@Service
public class EngineMaintenanceServiceImpl extends ServiceImpl<EngineMaintenanceMapper, Ruleengine>
        implements IEngineMaintenanceService
{
    @Autowired
    private EngineMaintenanceMapper engineMaintenanceMapper;

    @Override
    public List<Ruleengine> list(Ruleengine ruleengine) {
        LambdaQueryWrapper<Ruleengine> queryWrapper = new LambdaQueryWrapper<>();


        if (ObjUtils.notNull(ruleengine.getName())){
            queryWrapper.like(Ruleengine::getName, ruleengine.getName());
        }

        if (ObjUtils.notNull(ruleengine.getType())){
            queryWrapper.eq(Ruleengine::getType, ruleengine.getType());
        }

        if (ObjUtils.notNull(ruleengine.getIsActivate())){
            queryWrapper.eq(Ruleengine::getIsActivate, ruleengine.getIsActivate());
        }

        if (ObjUtils.notNull(ruleengine.getStatus())){
            queryWrapper.eq(Ruleengine::getStatus, ruleengine.getStatus());
        }
        if (ObjUtils.notNull(ruleengine.getRemark())){
            queryWrapper.eq(Ruleengine::getRemark, ruleengine.getRemark());
        }

        if (ObjUtils.notNull(ruleengine.getDescription())){
            queryWrapper.eq(Ruleengine::getDescription, ruleengine.getDescription());
        }

        if (ObjUtils.notNull(ruleengine.getCreateBy())){
            queryWrapper.eq(Ruleengine::getCreateBy, ruleengine.getCreateBy());
        }

        if (ObjUtils.notNull(ruleengine.getCreateTime())){
            queryWrapper.eq(Ruleengine::getCreateTime, ruleengine.getCreateTime());
        }

        if (ObjUtils.notNull(ruleengine.getUpdateBy())){
            queryWrapper.eq(Ruleengine::getUpdateBy, ruleengine.getUpdateBy());
        }

        if (ObjUtils.notNull(ruleengine.getUpdateTime())){
            queryWrapper.eq(Ruleengine::getUpdateTime, ruleengine.getUpdateTime());
        }
        return list(queryWrapper);
    }

    @Override
    public Result compiler(RuleContentReq ruleContentReq) {
        String className = ruleContentReq.getClassName();
        String content = ruleContentReq.getRuleContent();
        // 指定输出目录
        String targetDirectory = "D:\\work\\etl\\cloud-server\\muyu-modules\\muyu-rule\\muyu-rule-server\\src\\main\\java";

        JavaCompiler compiler = ToolProvider.getSystemJavaCompiler();
        try (StandardJavaFileManager fileManager = compiler.getStandardFileManager(null, null, null)) {
            // 创建一个内存中的源文件
            JavaFileObject sourceFileObject = new JavaSourceFromString(className, content);

            // 编译选项
            Iterable<String> options = Arrays.asList("-d", targetDirectory);

            // 编译源代码
            JavaCompiler.CompilationTask task = compiler.getTask(null, fileManager, null, options, null, Arrays.asList(sourceFileObject));
            boolean success = task.call();

            if (success) {
                return null;
            }

        } catch (Exception e) {
            Result.error(e.getMessage());
        }
        return null;
    }

    @Override
    public Result loader(RuleContentReq ruleContentReq) {
        String className = ruleContentReq.getClassName();
        try {
            // 假设我们有一个名为com.example.MyClass的类文件
            // 注意：这里应该是.class文件的路径
            String classPath = "D:\\work\\etl\\cloud-server\\muyu-modules\\muyu-rule\\muyu-rule-server\\src\\main\\java\\com\\muyu\\engine\\custom\\"+className+".class";
            // 读取类文件
            byte[] classData = Files.readAllBytes(Paths.get(classPath));

            // 提取类名（不包括.class和路径）
            String code = classPath.substring(classPath.lastIndexOf('\\') + 1).replace(".class", "");

            // 使用自定义类加载器加载类
            MyClassLoader classLoader = new MyClassLoader();
            Class<?> clazz = classLoader.defineClassFromBytes(code, classData);

            // 使用反射创建实例并调用方法
            Object instance = clazz.getDeclaredConstructor().newInstance();
            Method execute = clazz.getMethod("execute", String.class);
            System.out.println(execute.invoke(instance, "zhangsan"));
            System.out.println(execute.invoke(instance, ""));
            // ... 调用方法等操作

        } catch (Exception e) {
            e.printStackTrace();
        }

        return Result.success("加载成功");
    }

    @Override
    public Result<RuleContentModel> getRuleContent(Long ruleId) {
        if (ObjUtils.notNull(ruleId)){
            RuleContentModel ruleContentModel=engineMaintenanceMapper.getRuleContent(ruleId);
            return Result.success(ruleContentModel);
        }
        return null;
    }


    public static class MyClassLoader extends ClassLoader {
        public Class<?> defineClassFromBytes(String name, byte[] data) {
            return defineClass(name, data, 0, data.length);
        }
    }

    static class JavaSourceFromString extends SimpleJavaFileObject {
        final String code;

        JavaSourceFromString(String name, String code) {
            super(URI.create("string:///" + name.replace('.', '/') + Kind.SOURCE.extension), Kind.SOURCE);
            this.code = code;
        }

        @Override
        public CharSequence getCharContent(boolean ignoreEncodingErrors) {
            return code;
        }
    }

    @Autowired
    private EngineVersionMapper engineVersionMapper;
    @Override
    public Result<EngineVersionReq> details(Long id) {
        //查询出引擎的信息
        Ruleengine ruleengine = engineMaintenanceMapper.selectById(id);
        //获取到引擎的ID
        Long ruleId = ruleengine.getId();
        List<EngineVersion> engineVersions = engineVersionMapper.selectList(
                new LambdaQueryWrapper<EngineVersion>() {{
                    eq(EngineVersion::getRuleId, ruleId);
                }}
        );

        return Result.success(
                EngineVersionReq.builder()
                        .ruleengine(ruleengine)
                        .engineVersionList(engineVersions)
                        .build()
        );
    }

    @Override
    public Result<String> engineOnOff(Ruleengine ruleengine) {
        return Result.success(
                engineMaintenanceMapper.updateById(ruleengine)>0?"引擎激活状态修改成功":"引擎激活状态修改失败"
        );
    }

    @Override
    public Result<String> statusOnOff(Ruleengine ruleengine) {
        return Result.success(
                engineMaintenanceMapper.updateById(ruleengine)>0?"引擎状态操作成功":"引擎状态操作失败"
        );
    }
}
